@@ -26,122 +26,49 @@ Dependencies:
 
 import os
 import sys
-import asyncio
+import pdb
 import platform
 import datetime
+from time import sleep
 from configparser import ConfigParser
 import argparse
 import inspect
 import subprocess
+import time
+import threading
 import boto3
-from botocore.exceptions import ClientError, ProfileNotFound
-from pyaws import Colors
-from pyaws.utils import stdout_message, export_json_object
-from keyup.colormap import ColorMap
+from prompt_toolkit.shortcuts import ProgressBar
+from botocore.exceptions import ClientError
+from libtools import Colors
+from libtools import stdout_message
+from libtools.js import export_iterobject
+from pyaws.session import authenticated, boto3_session
 from keyup.statics import PACKAGE, local_config
 from keyup.help_menu import menu_body
-from keyup.progress import progress_bar as progress_timer
-from keyup import about, logger, keyconfig, __version__
 from keyup.map import map_identity, map_iam_username
 from keyup.list_ops import list_keys
 from keyup.iam_operations import local_profilenames
 from keyup.cauth import prepare_reportdata, setup_table
-from keyup.thread_progress import ProgressBarThread
+from keyup.progress_main import ProgressBarMain
+from keyup.thread_progress import custom_formatters, style
+from keyup import about, container, logger, keyconfig, __version__
+
+
+# global objects
+config = ConfigParser()
+c = Colors()
 
 
 try:
     from keyup.oscodes_unix import exit_codes
     os_type = 'Linux'
     splitchar = '/'                             # character for splitting paths (linux)
-    text = Colors.BRIGHT_CYAN
+    text = c.BRIGHT_CYAN
 except Exception:
     from keyup.oscodes_win import exit_codes    # non-specific os-safe codes
     os_type = 'Windows'
     splitchar = '\\'                            # character for splitting paths (windows)
-    text = Colors.CYAN
-
-
-# global objects
-config = ConfigParser()
-cm = ColorMap()
-
-# universal colors
-rd = Colors.RED
-act = Colors.ORANGE
-yl = Colors.YELLOW
-fs = Colors.GOLD3
-bd = Colors.BOLD
-frame = text
-bdwt = cm.bdwt
-rst = Colors.RESET
-
-
-def authenticated(profile):
-    """
-        Tests generic authentication status to AWS Account
-
-    Args:
-        :profile (str): iam user name from local awscli configuration
-    Returns:
-        TYPE: bool, True (Authenticated)| False (Unauthenticated)
-    """
-    try:
-        sts_client = boto3_session(service='sts', profile=profile)
-        httpstatus = sts_client.get_caller_identity()['ResponseMetadata']['HTTPStatusCode']
-        if httpstatus == 200:
-            return True
-
-    except ClientError as e:
-        if e.response['Error']['Code'] == 'InvalidClientTokenId':
-            logger.info(
-                '%s: Invalid credentials to authenticate for profile user (%s). Exit. [Code: %d]'
-                % (inspect.stack()[0][3], profile, exit_codes['EX_NOPERM']['Code']))
-        elif e.response['Error']['Code'] == 'ExpiredToken':
-            logger.info(
-                '%s: Expired temporary credentials detected for profile user (%s) [Code: %d]'
-                % (inspect.stack()[0][3], profile, exit_codes['EX_CONFIG']['Code']))
-        else:
-            logger.exception(
-                '%s: Unknown Boto3 problem. Error: %s' %
-                (inspect.stack()[0][3], e.response['Error']['Message']))
-    except Exception as e:
-        return False
-    return False
-
-
-def boto3_session(service, profile=None):
-    """
-        Establishes boto3 sessions, client
-
-    Args:
-        :service (str): boto3 service abbreviation ('ec2', 's3', etc)
-        :profile (str): profile_name of an iam user from local awscli config
-    Returns:
-        TYPE: boto3 client object
-    """
-    try:
-        if profile:
-            if profile == 'default':
-                client = boto3.client(service)
-            else:
-                session = boto3.Session(profile_name=profile)
-                client = session.client(service)
-        else:
-            client = boto3.client(service)
-    except ClientError as e:
-        logger.exception(
-            "%s: IAM user or role not found (Code: %s Message: %s)" %
-            (inspect.stack()[0][3], e.response['Error']['Code'],
-             e.response['Error']['Message']))
-        raise
-    except ProfileNotFound:
-        msg = (
-            '%s: The profile (%s) was not found in your local config. Exit.' %
-            (inspect.stack()[0][3], profile))
-        stdout_message(msg, 'FAIL')
-        logger.warning(msg)
-        sys.exit(exit_codes['EX_NOUSER']['Code'])
-    return client
+    text = c.CYAN
 
 
 def help_menu():
@@ -149,8 +76,8 @@ def help_menu():
     Displays help menu contents
     """
     print(
-        Colors.BOLD + Colors.BRIGHT_WHITE +
-        '\n\t'.expandtabs(27) + PACKAGE + Colors.RESET + ' command help'
+        c.BOLD + c.BRIGHT_WHITE +
+        '\n\t'.expandtabs(27) + PACKAGE + c.RESET + ' command help'
         )
     print(menu_body)
     return
@@ -179,18 +106,30 @@ def get_current_key(profile_name, surrogate=''):
         key_id = subprocess.getoutput(cmd)
     except Exception as e:
         logger.exception(
-            '%s: Failed to identify AccessKeyId used in %s profile.' %
-            (inspect.stack()[0][3], profile_name))
+            '%s: Failed to identify AccessKeyId used in %s profile. Error: %s' %
+            (inspect.stack()[0][3], profile_name, str(e)))
         return ''
     return key_id
 
 
-def remove_temporary_credentials(config_object):
-    """Removes expired temporary credentials from configparser config obj """
-    clean_profile_list = []
-    prefix = ''
-    if prefix:
-        clean_profile_list = list(filter(lambda x: prefix not in x, config_object.sections()))
+def remove_temporary_credentials(config_object, prefix='', profiles=False):
+    """
+        Filers temporary credentials from the list of profilenames
+        present in the local awscli configuration
+
+    Args:
+        :config_object (configParser object): object containing profilenames
+            from the local awscli credentials file
+        :prefix (str):  any prefix prepended to profilenames to indicate temporary
+            role credentials in the local awscli credentials file
+        :profiles (bool): flag indicating return of a profilename list instead of
+
+    Returns:
+        configParser object | profilename list (list)
+    """
+    if prefix and profiles:
+        clean_profiles = list(filter(lambda x: prefix not in x, config_object.sections()))
+        return clean_profiles
     else:
         for profile in config_object.sections():
             if 'aws_security_token' in config_object[profile].keys():
@@ -337,7 +276,9 @@ def delete_keyset(access_key, profile, surrogate=''):
 
     Args:
         - **access_key (str)**:  AccessKeyId of the keyset to delete
-        - **user (str)**: profile name of iam user from which to delete key
+        - **profile (str)**: iam user alias in the local awscli config
+        - **surrogate (str)**: iam username on which access key operations
+            are conducted by another iam user denoted in profile
     Returns:
         TYPE: bool, Success | Failure
     """
@@ -387,12 +328,23 @@ def delete_keyset(access_key, profile, surrogate=''):
 
 
 def create_keyset(iam_user, profile, surrogate=''):
-    """ Creates new access key, secret key pair for iam user """
+    """
+        Creates new access key, secret key pair for iam user
+
+    Args:
+        - **access_key (str)**:  AccessKeyId of the keyset to delete
+        - **profile (str)**: iam user alias in the local awscli config
+        - **surrogate (str)**: iam username on which access key operations
+            are conducted by another iam user denoted in profile
+
+    Returns:
+        Success | Failure, TYPE: bool, aws access keys (dict)
+    """
     try:
         logger.info(
-            'Request to create new keyset for %s user %s' %
-            ('surrogate' if surrogate else 'iam',
-             surrogate if surrogate else iam_user)
+                'Request to create new keyset for %s user %s' %
+                ('surrogate' if surrogate else 'iam',
+                 surrogate if surrogate else iam_user)
             )
         # create keyset for primary iam user unless for a surrogate
         client = boto3_session(service='iam', profile=profile)
@@ -403,7 +355,7 @@ def create_keyset(iam_user, profile, surrogate=''):
     except ClientError as e:
         if e.response['Error']['Code'] == 'AccessDenied':
             logger.warning(
-                'IAM user does not have permissions to create new access keys'
+                'IAM profile user does not have permissions to create new access keys'
                 )
             logger.warning(exit_codes['EX_NOPERM']['Reason'])
             sys.exit(exit_codes['EX_NOPERM']['Code'])
@@ -519,8 +471,8 @@ def write_keyset_backup(keys, user, quiet):
         Writes newly created keyset to disk provided configuration file flag set:
 
     Args:
-        keys:  New keyset object
-        quiet: When set, supresses all output to stdout
+        :keys:  New keyset object
+        :quiet: When set, supresses all output to stdout
     Returns:
         TYPE: bool, Success | Failure
     """
@@ -538,14 +490,14 @@ def write_keyset_backup(keys, user, quiet):
     fs_location = local_config['KEY_BACKUP']['BACKUP_LOCATION']
     filename = date + '-' + user + '-accessKeys.json'
     output_file = fs_location + splitchar + filename
-    success_msg = 'Copy of new keyset written to: %s' % (Colors.BOLD + Colors.WHITE + output_file + Colors.RESET)
+    success_msg = 'Copy of new keyset written to: %s' % (c.BOLD + c.WHITE + output_file + c.RESET)
     fail_msg = 'Problem writing new keyset to backup location: %s' % output_file
     try:
         # write keyset
         if os.path.exists(fs_location):
             if os.path.exists(output_file):
                 logger.info('%s pre-existing object - overwriting file' % output_file)
-            r = export_json_object(safe_keyset(keys), output_file)
+            r = export_iterobject(safe_keyset(keys), output_file)
         else:
             logger.warning('Directory location to store new keysets (%s) does not exist' % fs_location)
             return False
@@ -575,7 +527,8 @@ def configure_keyset(keyset, profile, surrogate=''):
     Args:
         :keyset (json): access keys, newly created
         :profile (str): iam user alias in the local awscli config
-        :surrogate (str): iam username creating keys to another iam user account
+        :surrogate (str): iam username on which access key operations
+            are conducted by another iam user denoted in profile
     Returns:
         :parsed (configParser):  object configured with new access key signatures
         :configfile_path (str):  os dependent path to awscli credentials file
@@ -656,7 +609,7 @@ def main(operation, profile, auto, debug, user_name=''):
         return True
 
     elif not operation:
-        msg_accent = (Colors.BOLD + 'list' + Colors.RESET + ' | ' + Colors.BOLD + 'up' + Colors.RESET)
+        msg_accent = (c.BOLD + 'list' + c.RESET + ' | ' + c.BOLD + 'up' + c.RESET)
         msg = """You must provide a valid OPERATION for --operation parameter:
 
                 --operation { """ + msg_accent + """ }
@@ -675,7 +628,23 @@ def main(operation, profile, auto, debug, user_name=''):
         # -- Key Rotation: 1 keyset exists -------------------------------------
         if len(keylist) == 1:       # 1 keyset exists
             deprecated_access_key = key_metadata[0]['AccessKeyId']
-            result, new_keys = create_keyset(iam_user=user, profile=profile, surrogate=user_name)
+
+            # delete keyset, must supply profile ------------------------------
+            r = delete_keyset(
+                    access_key=deprecated_access_key,
+                    profile=profile,
+                    surrogate=user_name
+                )
+            if r:
+                result, new_keys = create_keyset(iam_user=user, profile=profile, surrogate=user_name)
+            else:
+                logger.warning(
+                    '%s: Deprecated access key %s may not have \
+                    been deleted properly. Check User Permissions' %
+                    (inspect.stack()[0][3], deprecated_access_key)
+                )
+                sys.exit(exit_codes['EX_DELETE_FAIL']['Code'])
+
             if result:
                 # assumble key
                 parsed, output_file, access_key, secret_key = configure_keyset(new_keys, profile, surrogate=user_name)
@@ -689,12 +658,45 @@ def main(operation, profile, auto, debug, user_name=''):
                 logger.exception(
                     '%s: New keys did not generate correctly. Keys received: %s\n\nAbort.' %
                     (inspect.stack()[0][3], str(new_keys)))
+
             # switch to new keys in mem before rewriting the credentials file
             if write_keyset(parsed, output_file, debug):
                 if set_keyset(access_key, secret_key):
                     # delete keyset, no profile given, use in memory keys
-                    progress_timer(KEY_ENABLE_DELAY, '  Key activation: ', quiet=auto)
-                    r = delete_keyset(access_key=deprecated_access_key, profile=profile, surrogate=user_name)
+                    d = 0.05     # duration / cycle (secs)
+                    print('\n')
+                    title = 'Rotating access keys... Please wait '
+                    iterations =  int(KEY_ENABLE_DELAY / d)
+                    exit_event = threading.Event()
+
+                    with ProgressBar(style=style, formatters=custom_formatters) as pb:
+                        # Two parallel tasks.
+                        def display_progress():
+                            for i in pb(range(iterations), label=title):
+                                time.sleep(d)
+                                if exit_event.is_set():
+                                    break
+                            sys.stdout.flush()
+
+                        def delay():
+                            sleep(KEY_ENABLE_DELAY)
+
+                        # Start threads.
+                        t1 = threading.Thread(target=display_progress)
+                        t2 = threading.Thread(target=delay)
+
+                        t1.daemon = True
+                        t2.daemon = True
+                        print('\n')
+                        t1.start()
+                        t2.start()
+
+                        # Wait for the primary work thread to finish (t2)
+                        while t2.is_alive():
+                            t2.join(timeout=0.5)        # wait only for report data thread
+                        else:
+                            exit_event.set()
+
                     keylist, key_metadata = list_keys(
                             account=aws_account,
                             profile=profile,
@@ -710,7 +712,8 @@ def main(operation, profile, auto, debug, user_name=''):
                                 user=user_name or profile,
                                 quiet=auto
                             )
-                    return r
+
+                    return True
                 else:
                     logger.warning(
                         '%s: Problem setting new keyset; failed authentication test (AccessKeyId: %s). Exit.' %
@@ -788,22 +791,55 @@ def main(operation, profile, auto, debug, user_name=''):
 
             # write new awscli config
             if write_keyset(parsed, output_file, debug):
-                if progress_timer(KEY_ENABLE_DELAY, '  Key activation: ', quiet=auto):
-                    keylist, key_metadata = list_keys(
-                            account=aws_account,
-                            profile=profile,
-                            iam_user=user,
-                            surrogate=user_name,
-                            stage='AFTER ROTATION',
+                d = 0.05     # duration / cycle (secs)
+                title = 'Rotating access keys... '
+                iterations =  int(KEY_ENABLE_DELAY / d)
+                exit_event = threading.Event()
+
+                with ProgressBar(style=style, formatters=custom_formatters) as pb:
+                    # Two parallel tasks.
+                    def display_progress():
+                        for i in pb(range(iterations), label=title):
+                            time.sleep(d)
+                            if exit_event.is_set():
+                                break
+                        sys.stdout.flush()
+
+                    def delay():
+                        sleep(KEY_ENABLE_DELAY)
+
+                    # Start threads.
+                    t1 = threading.Thread(target=display_progress)
+                    t2 = threading.Thread(target=delay)
+
+                    t1.daemon = True
+                    t2.daemon = True
+                    print('\n')
+                    t1.start()
+                    t2.start()
+
+                    # Wait for the primary work thread to finish (t2)
+                    while t2.is_alive():
+                        t2.join(timeout=0.5)        # wait only for report data thread
+                    else:
+                        exit_event.set()
+
+                #print('\n')
+                keylist, key_metadata = list_keys(
+                        account=aws_account,
+                        profile=profile,
+                        iam_user=user,
+                        surrogate=user_name,
+                        stage='AFTER ROTATION',
+                        quiet=auto
+                    )
+                # write copy of new keyset to backup location if config file flag set
+                if local_config['KEY_BACKUP']['BACKUP_ENABLE']:
+                    write_keyset_backup(
+                            keys=new_keys['AccessKey'],
+                            user=user_name or profile,
                             quiet=auto
                         )
-                    # write copy of new keyset to backup location if config file flag set
-                    if local_config['KEY_BACKUP']['BACKUP_ENABLE']:
-                        write_keyset_backup(
-                                keys=new_keys['AccessKey'],
-                                user=user_name or profile,
-                                quiet=auto
-                            )
                 return True
             else:
                 logger.warning(
@@ -839,10 +875,10 @@ def options(parser, help_menu=False):
                               required=False, help="type (default: %(default)s)")
     parser.add_argument("-o", "--operation", nargs='?', default='list', type=str,
                         choices=VALID_OPERATIONS, required=False)
-    parser.add_argument("-a", "--auto", dest='auto', action='store_true', required=False)
     parser.add_argument("-c", "--configure", dest='configure', action='store_true', required=False)
     parser.add_argument("-d", "--debug", dest='debug', action='store_true', required=False)
     parser.add_argument("-h", "--help", dest='help', action='store_true', required=False)
+    parser.add_argument("-q", "--quiet", dest='quiet', action='store_true', required=False)
     parser.add_argument("-R", "--key-report", dest='keyreport', action='store_true', required=False)
     parser.add_argument("-u", "--user-name", dest='username', type=str, required=False)
     parser.add_argument("-V", "--version", dest='version', action='store_true', required=False)
@@ -892,17 +928,7 @@ def source_globals():
     KEYAGE_WARNING = local_config['KEY_METADATA']['KEYAGE_WARNING']
 
 
-async def async_caller():
-    """Calls asynchronous functions"""
-    #os.system('cls' if os.name == 'nt' else 'clear')       # clear screen
-    print('\n\n')
-    result2 = loop.create_task(ptk_main())
-    result1 = loop.create_task(prepare_reportdata())
-    await asyncio.wait([result1, result2])
-    return result1
-
-
-def init_cli():
+def init():
     """Caller function; initializes all functionality"""
     try:
 
@@ -951,13 +977,35 @@ def init_cli():
         d = 0.05     # duration / cycle (secs)
         title = 'Querying Amazon STS'
         iterations = round(len(local_profilenames()) / d)
-        pb_thread = ProgressBarThread(label=title, delay=d, cycles=iterations)
-        pb_thread.start()
+        exit_event = threading.Event()
 
-        data, skipped = prepare_reportdata()
-        if data:
-            pb_thread.stop()
-            setup_table(data, skipped)
+        with ProgressBar(style=style, formatters=custom_formatters) as pb:
+            # Two parallel tasks.
+            def display_progress():
+                for i in pb(range(iterations), label=title):
+                    time.sleep(d)
+                    if exit_event.is_set():
+                        break
+                sys.stdout.flush()
+
+            # Start threads.
+            t1 = threading.Thread(target=display_progress)
+            t2 = threading.Thread(target=prepare_reportdata, args=(container, ))
+
+            t1.daemon = True
+            t2.daemon = True
+            t1.start()
+            t2.start()
+
+            # Wait for the primary work thread to finish (t2)
+            while t2.is_alive():
+                t2.join(timeout=0.5)        # wait only for report data thread
+            else:
+                exit_event.set()
+
+        # Exit ProgessBar context & unpack Queue
+        data, skipped = container.get()
+        setup_table(data, skipped)
 
     else:
         if precheck():              # if prereqs set, run
@@ -967,9 +1015,9 @@ def init_cli():
                             operation=args.operation,
                             profile=args.profile,
                             user_name=args.username,
-                            auto=args.auto,
+                            auto=args.quiet,
                             debug=args.debug
-                            )
+                        )
                 if success:
                     logger.info('IAM access keyset operation complete')
                     sys.exit(exit_codes['EX_OK']['Code'])
